#include "globals.h"

#define XtNdevices	"devices"
#define XtCDevices	"Devices"

typedef struct
{
  String deviceRules;
} DevResData;

static XtResource resources[] = {
  {
        XtNdevices,
        XtCDevices,
        XtRString,
        sizeof( String ),
        XtOffsetOf( DevResData, deviceRules ),
        XtRString,
        (XtPointer)""
  }
};

static DevResData devResData;
static struct Device *devices;
extern struct FileManager *fmgr;


void readDevApplicationResources( toplevel )
  Widget toplevel;
{
  XtVaGetApplicationResources( toplevel, &devResData, resources,
        XtNumber(resources), NULL );
}

void parseDevApplicationResources( w, fg, bg )
  Widget w;
  Pixel fg, bg;
{
  char *ptr = devResData.deviceRules;
  char buffer[NAME_LENGTH];

  while( sscanf( ptr, "%[^\n]\n", buffer ) == 1 )
  {
    char *tmpPtr, name[256], command[256], pixmapName[256];
    Pixmap p;
    struct Device *dev;
    XmString xms;
    Widget form, label, nameLabel;

    if( sscanf( buffer, "%s %s", name, pixmapName ) != 2 )  continue;

    if( (p = isaDevice( pixmapName )) == XmUNSPECIFIED_PIXMAP )
    {
      p = loadPixmap( w, pixmapName, fg, bg );
    }

    if( p != XmUNSPECIFIED_PIXMAP )
    {
      dev = appendDevice( &devices );
      strcpy( dev->name, name );
      strcpy( dev->pixmapName, pixmapName );
      dev->p = p;
      strcpy( dev->command, strchr( buffer, '\"' ) + 1 );
      tmpPtr = strchr( dev->command, '\"' );
      *tmpPtr = '\0';
    }

    if( strlen(buffer)+1 >= strlen(ptr) )
      break;
    else
      ptr += strlen( buffer ) + 1;

    memset( (void *)buffer, '\0', sizeof(buffer) );
  }

  createDeviceIcons( w, fg, bg );
}

void loadDevicesData( w )
  Widget w;
{
  int i, j, n = numDevices(devices), max1 = 0, max2 = 0;
  char buffer[256];

  for( i = 0; i < n; i++ )
  {
    struct Device *dev = getIthDevice( devices, i );
    if( strlen(dev->name) > max1 )  max1 = strlen(dev->name);
    if( strlen(dev->pixmapName) > max2 )  max2 = strlen(dev->pixmapName);
  }

  max1 += 3;
  max2 += 3;

  XmListDeleteAllItems( w );
  for( i = 0; i < n; i++ )
  {
    XmString xms;
    struct Device *dev = getIthDevice( devices, i );
    strcpy( buffer, dev->name );
    for( j = strlen(dev->name); j < max1; j++ )  strcat( buffer, " " );
    strcat( buffer, dev->pixmapName );
    for( j = strlen(dev->pixmapName); j < max2; j++ )  strcat( buffer, " " );
    strcat( buffer , "\"" );
    strcat( buffer, dev->command );
    strcat( buffer , "\"" );
    xms = XmStringCreateLtoR( buffer, "set1" );
    XmListAddItemUnselected( w, xms, 0 );
    XmStringFree( xms );
  } 
}

void writeDeviceResources( fd )
  FILE *fd;
{
  int i, n = numDevices(devices);

  if( n > 0 )  fprintf( fd, "mfm*devices: %s\n", "\\" );
  for( i = 0; i < n; i++ )
  {
    struct Device *dev = getIthDevice( devices, i );
    fprintf( fd, "\t%s\t%s\t\"%s\"", dev->name,
	dev->pixmapName, dev->command );

    if( i != n-1 )  
	fprintf( fd, " %s%s%s\n", "\\", "n", "\\" );
    else
	fprintf( fd, "\n" );
  }
}

void insertDev( w1, w2, w3 )
  Widget w1, w2, w3;
{
  char *name, *command, *pixmapName;
  Pixmap p;
  Pixel fg, bg;

  XtVaGetValues( XtParent(w1), XmNforeground, &fg,
	XmNbackground, &bg, NULL );

  name = XmTextFieldGetString( w1 );
  pixmapName = XmTextFieldGetString( w2 );
  command = XmTextFieldGetString( w3 );

  if( !strcmp( name, "" ) || !strcmp( pixmapName, "" ) || !strcmp( command, "" ) )
  {
    popupErrorDialog( "All Three Device Fields Must Have Values..." );
    XtFree( name );
    XtFree( command );
    XtFree( pixmapName );
    return;
  }

  if( (p = isaDevice( pixmapName )) == XmUNSPECIFIED_PIXMAP )
  {
    p = loadPixmap( XtParent(w1), pixmapName, fg, bg );
  }

  if( p != XmUNSPECIFIED_PIXMAP )
  {
    struct Device *dev = appendDevice( &devices );
    strcpy( dev->name, name );
    strcpy( dev->pixmapName, pixmapName );
    dev->p = p;
    strcpy( dev->command, command );
  }
  else
  {
    char errorString[256];
    sprintf( errorString, "Cannot Load Pixmap %s", pixmapName );
    popupErrorDialog( errorString );
    XtFree( name );
    XtFree( command );
    XtFree( pixmapName );
    return;
  }

  XmTextFieldSetString( w1, "" );
  XmTextFieldSetString( w2, "" );
  XmTextFieldSetString( w3, "" );

  XtFree( name );
  XtFree( command );
  XtFree( pixmapName );
}

void deleteDevice( i )
  int i;
{
  removeIthDevice( &devices, i );
}

void execDeviceCommand( w, d, n, s )
  Widget w;
  struct Device *d;
  int n;
  char *s;
{
  if( strstr( s, "$ALL" ) )
  {
    char *cmd = (char *)XtMalloc( strlen(s) + strlen(d->command) + 5 );
    strcpy( cmd, d->command );
    insertFileName( s, cmd );
    if( strcmp( cmd, "" ) )  launchCmd( w, cmd );
    XtFree( (char *)cmd );
  }
  else
  {
    char *ptr = s, buffer[256];
    int i = 0;

    while( i < n && sscanf( ptr, "%s", buffer ) == 1 )
    {
      char *cmd = (char *)XtMalloc( strlen(buffer) + strlen(d->command) + 5 );
      strcpy( cmd, d->command );
      insertFileName( buffer, cmd );
      if( strcmp( cmd, "" ) )  
      {
	launchCmd( w, cmd );
        return;
      }
      XtFree( (char *)cmd );

      ptr += strlen( buffer );
      while( ptr[0] == ' ' )  ptr++;
      memset( (void *)buffer, '\0', sizeof(buffer) );
    }
  }
}

void createDeviceIcons( w, fg, bg )
  Widget w;
  Pixel fg, bg;
{
  int i, n = numDevices(devices);
  Widget form, label, nameLabel;
  XmString xms;

  for( i = 0; i < n; i++ )
  {
    struct Device *dev = getIthDevice( devices, i );

    form = XtVaCreateManagedWidget( "form",
                xmFormWidgetClass, w,
                XmNwidth, 60,
                XmNheight, 85,
                XmNforeground, fg,
                XmNbackground, bg,
                XmNuserData, (XtPointer)dev,
                NULL );
    label = XtVaCreateManagedWidget( "label",
                xmDrawnButtonWidgetClass, form,
                XmNlabelType, XmPIXMAP,
                XmNlabelPixmap, dev->p,
                XmNtopAttachment, XmATTACH_FORM,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 25,
                XmNleftAttachment, XmATTACH_FORM,
                XmNrightAttachment, XmATTACH_FORM,
                XmNforeground, fg,
                XmNbackground, bg,
                XmNshadowThickness, 0,
                XmNuserData, (XtPointer)dev,
                NULL );
    registerDropSite( label, DEV_DROP_SITE );
    unregisterDragSite( label );
    xms = XmStringCreateSimple( dev->name );
    nameLabel = XtVaCreateManagedWidget( "nameLabel",
                xmLabelWidgetClass, form,
                XmNlabelString, xms,
                XmNtopAttachment, XmATTACH_WIDGET,
                XmNtopWidget, label,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNleftAttachment, XmATTACH_FORM,
                XmNrightAttachment, XmATTACH_FORM,
                XmNforeground, fg,
                XmNbackground, bg,
                NULL );
    unregisterDragSite( nameLabel );
    XmStringFree( xms );
  }
}
